#!/bin/env ruby

# frozen_string_literal: true

require 'strscan'

class StringScanner
  def scan_nonil(regexp)
    res = scan(regexp)
    if res
      return res
    else
      return ""
    end
  end
end

lines = {}

rb_inner_hdr_dir = File.join(RbConfig::CONFIG["rubyhdrdir"], "ruby")
Dir.chdir(rb_inner_hdr_dir)
Dir::glob('**/*.h').each do |fpath|
  fname = File.basename(fpath)
  lines[fname] = []

  h_file = File.open(File.join(rb_inner_hdr_dir, fpath))
  h_file.each_line do |line|
    str = ""
    s = StringScanner.new(line)
    str += s.scan_nonil(/(VALUE|#define) rb_[a-z0-9_]+_new[a-z0-9_]*/)
    if !str.empty?
      if str =~ /#define/
        str += s.scan_nonil(/ +[a-zA-Z0-9_]+/)
      end
      str += s.scan_nonil(/\([^)]+\)/)
      lines[fname] << str
    end
  end
  h_file.close
end

lines.filter! {|key,value| !value.empty?}

longest_h_name_len = lines.keys.max {|a,b| a.length <=> b.length}.length

fmted_lines = []

lines.each_pair do |fname, lines_arr|
  line_set = []

  h_name_padding = " "*(longest_h_name_len - fname.length + 1)
  h_name_fmted = fname + ":" + h_name_padding
  lines_arr.each do |l|
    l.sub!("VALUE", "VALUE" + " "*3)
    l.sub!("define", "define" + " ")
    l.rstrip!
    line_set << h_name_fmted + l + "\n"
  end

  fmted_lines << line_set
end

newlines = []
(fmted_lines.length - 1).times do
  newlines << "\n"
end

print fmted_lines.zip(newlines).flatten.compact.join
